from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app import schemas
from app.api.deps import get_current_active_superuser, get_db
from app.services import product_service

router = APIRouter()

@router.get("/", response_model=List[schemas.Product])
def read_products(
    db: Session = Depends(get_db),
    skip: int = 0,
    limit: int = 100,
) -> Any:
    """
    Retrieve products.
    """
    products = product_service.get_multi(db, skip=skip, limit=limit)
    return products

@router.post("/", response_model=schemas.Product)
def create_product(
    *,
    db: Session = Depends(get_db),
    product_in: schemas.ProductCreate,
    current_user: schemas.User = Depends(get_current_active_superuser),
) -> Any:
    """
    Create new product.
    """
    product = product_service.create(db, obj_in=product_in)
    return product

@router.get("/{id}", response_model=schemas.Product)
def read_product(
    *,
    db: Session = Depends(get_db),
    id: int,
) -> Any:
    """
    Get product by ID.
    """
    product = product_service.get(db, id=id)
    if not product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Product not found",
        )
    return product

@router.put("/{id}", response_model=schemas.Product)
def update_product(
    *,
    db: Session = Depends(get_db),
    id: int,
    product_in: schemas.ProductUpdate,
    current_user: schemas.User = Depends(get_current_active_superuser),
) -> Any:
    """
    Update a product.
    """
    product = product_service.get(db, id=id)
    if not product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Product not found",
        )
    product = product_service.update(db, db_obj=product, obj_in=product_in)
    return product

@router.delete("/{id}", response_model=schemas.Product)
def delete_product(
    *,
    db: Session = Depends(get_db),
    id: int,
    current_user: schemas.User = Depends(get_current_active_superuser),
) -> Any:
    """
    Delete a product.
    """
    product = product_service.get(db, id=id)
    if not product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Product not found",
        )
    product = product_service.remove(db, id=id)
    return product
